/*
**  Copyright 2001-2003  University of Illinois Board of Trustees
**  Copyright 2001-2003  Mark D. Roth
**  All rights reserved.
**
**  mkpwdbm.c - create DBM files for pam_dbm
**
**  Mark D. Roth <roth@uiuc.edu>
**  Campus Information Technologies and Educational Services
**  University of Illinois at Urbana-Champaign
*/

#include <config.h>

#include <stdio.h>
#include <fcntl.h>

#ifdef STDC_HEADERS
# include <string.h>
# include <stdlib.h>
#endif

#if HAVE_UNISTD_H
# include <unistd.h>
#endif

#if HAVE_NDBM_H
# include <ndbm.h>
#elif HAVE_GDBM_NDBM_H
# include <gdbm/ndbm.h>
#endif

#include <compat.h>


static char *progname;


static void
usage(void)
{
	printf("Usage: %s [-mvVq] [-F delimiters] dbmfile\n", progname);
	exit(1);
}


int
main(int argc, char *argv[])
{
	char *dbm_file;
	DBM *dbm;
	datum keydat, valdat;
	char *delim = ":";
	char *cp1, *cp2, *nextp;
	char input[1024], buf[1024];
	unsigned long ct = 0;
	unsigned short verbose = 1;
	unsigned short merge = 0;
	int i;

	progname = basename(argv[0]);

	while ((i = getopt(argc, argv, "F:hmvVq")) != -1)
	{
		switch (i)
		{
		case 'F':
			delim = optarg;
			break;
		case 'm':
			merge = 1;
			break;
		case 'V':
			fprintf(stderr, "%s-" PACKAGE_VERSION "\n", progname);
			exit(0);
		case 'v':
			verbose++;
			break;
		case 'q':
			verbose = 0;
			break;
		case 'h':
		default:
			usage();
		}
	}

	if (argc - optind != 1)
		usage();
	dbm_file = argv[optind];

	if (verbose > 0)
		printf("%s: %s %s...\n", progname,
		       (merge ? "merging into" : "creating"), dbm_file);

	dbm = dbm_open(dbm_file, O_WRONLY|O_CREAT|(merge ? 0 : O_EXCL), 0600);
	if (dbm == NULL)
	{
		perror("dbm_open()");
		exit(1);
	}

	while (fgets(input, sizeof(input), stdin) != NULL)
	{
		/* strip newlines */
		if (input[strlen(input) - 1] == '\n')
			input[strlen(input) - 1] = '\0';

		strlcpy(buf, input, sizeof(buf));
		nextp = buf;
		if ((cp1 = strsep(&nextp, delim)) == NULL
		    || (cp2 = strsep(&nextp, delim)) == NULL
		    || *cp1 == '\0'
		    || *cp2 == '\0')
		{
			fprintf(stderr, "%s: bad input record: \"%s\""
				" - skipping\n", progname, input);
			continue;
		}

		keydat.dptr = cp1;
		keydat.dsize = strlen(cp1) + 1;
		valdat.dptr = cp2;
		valdat.dsize = strlen(cp2) + 1;

		if (verbose > 1)
			printf("record %06lu: key=\"%s\" value=\"%s\"\n",
			       (ct + 1), cp1, cp2);

		if (dbm_store(dbm, keydat, valdat,
			      (merge ? DBM_REPLACE : DBM_INSERT)) < 0)
		{
			perror("dbm_store()");
			exit(1);
		}

		ct++;
	}

	if (!feof(stdin))
	{
		perror("fgets()");
		exit(1);
	}

	dbm_close(dbm);
	if (verbose > 0)
		printf("%s: %lu records written to %s\n",
		       progname, ct, dbm_file);
	exit(0);
}


